@@ -26,71 +26,34 @@ import pandas as pd
 
 import traceback
 import requests
-
+#from ws4py.client.threadedclient import WebSocketClient
 import threading
 
 import websocket
 from copy import deepcopy
+import time
 
 
-appendIpAddress = "192.168.1.1"
-# appendIpAddress = None
+from os.path import dirname, abspath
 
-# REAL ENV.
-KAISA_ROOT_URL = "kgl.jt00000.com"
-KAISA_ROOT_URL_SIM = "sit-kgl.jt00000.com"
-# KAISA_ROOT_URL = KAISA_ROOT_URL_SIM
-KAISA_ROOT_URL_API = "openapi.jt00000.com"
-#
-CRYPTO_RSA_URL = "https://__kaisarooturl__/kgl-third-authorization/crypto/key/RSA"
-AUTHENTICA_URL = "https://__kaisarooturl__/kgl-third-authorization/oauth/token"
-# AUTHENTICA_URL = "https://openapi.jt00000.com"
-QUOTE_URL = "https://kgl.jt00000.com/hq"
 
-# WEBSOCKET_DATA_HOST = "ws://__kaisarooturl__/dz_app_ws/ws"
-WEBSOCKET_DATA_HOST = "ws://sit-kgl.jt00000.com/dz_app_ws/ws"
-REST_DATA_HOST = "https://__kaisarooturl__/dzApp/dzHttpApi"
+project_path = dirname(dirname(abspath(__file__)))
+projec_full_path = project_path + r'\common'
+sys.path.append(projec_full_path)
 
-TRADE_CRYPTO_RSA_URL = "https://__kaisarooturl__/kgl-trade-service/crypto/key/RSA"
-ClientByMobile_URL = "https://__kaisarooturl__/kgl-user-center/userOaccAccount/selectClientIdByMobile"
-REST_HOST = "https://__kaisarooturl__/kgl-trade-service"
-WEBSOCKET_TRADE_HOST = "wss://__kaisarooturl__/kgl-trade-push-service/ws"
 
-openapi_scope = "wthk"
-openapi_scope = "read"
-
-MARKET_POST_URL = "https://sit-kgl.jt00000.com/kgl-stock-push-provider"
-MARKET_WS_URL = "ws://uat-stock.jt00000.com/stock"
-
-from kaisaglobalapi.common import KaisaCrypto
-from kaisaglobalapi.common.utils import *
-
-from kaisaglobalapi.common import KaisaProtoc
+from com import *
+from KaisaCrypto import *
+from utils import *
+from KaisaProtoc import *
 
 kcrypto = KaisaCrypto()
 kprotoc = KaisaProtoc()
 
-START_PUSH = 200
-STOP_PUSH = 201
-SYNCHRONIZE_PUSH = 250
-QUERY_HISTORY = 36
-QUERY_CONTRACT = 52
-ON_TICK = 251
-ON_MARKETDATA = 153
-PING = 2
-PONG = 3
-LOGIN = 10
-HKSE_MARKET = 2002
-
-CREATION = 101
-UPDATE = 102
-TRADE = 103
-CANCELLATION = 104
-ACCOUT = 106
-POSITION = 105
 
 
-class KaisaGateway():
+
+class KaisaGateway(object):
 
     req_id = 0
 
@@ -116,6 +79,9 @@ class KaisaGateway():
 
     _ws_trade_connect_status = False
 
+    _market_connect_msg = ""
+
+
     def __init__(self):
         pass
 
@@ -167,7 +133,7 @@ class KaisaGateway():
         """
 
         if self.gateway_auth_status:
-            return self.gateway_auth_status
+            return self.gateway_auth_status, self.gateway_auth_msg
 
         # only for token of gateway.
         timestamp_ = generate_timestamp()
@@ -187,11 +153,10 @@ class KaisaGateway():
         if ipAddress:
             params["ipAddress"] = ipAddress
 
-        headerParam = {"ipAddress":ipAddress,"phoneModel":"","deviceId":"","deviceToken":"","macAddress":"","channelId":"","country":"","sysVersion":"","apiVersion":"","platform":""}
+        headerParam = {"srvType":"open","ipAddress":ipAddress,"phoneModel":"","deviceId":"","deviceToken":"","macAddress":"","channelId":"","country":"","sysVersion":"","apiVersion":"","platform":""}
         headerParam = json.dumps(headerParam)
 
         authorization_str = "basic {}".format(self.openapi_token)
-        # print(authorization_str)
         headers = {
             "Authorization": authorization_str,
             "Content-Type": "application/x-www-form-urlencoded",
@@ -223,7 +188,6 @@ class KaisaGateway():
             self.write_log("网关认证失败")
             self.write_error(data)
 
-
         return self.gateway_auth_status, self.gateway_auth_msg
 
     def gateway_auth(self):
@@ -232,7 +196,7 @@ class KaisaGateway():
     def market_connect_ws(self):
         ret_ok, ret_msg = self.gateway_auth()
         if ret_ok:
-            self.market_connect_ws()
+            self.market_connect_ws0()
         time.sleep(5)
 
     def do_market_requests(self, url, reqdata):
@@ -254,7 +218,6 @@ class KaisaGateway():
         resp_data = response.json()
         return status_code, resp_data
 
-
     def do_trade_requests(self, url, reqdata, reqtype="normal"):
 
         reqdata = self._trade_req_decorate(reqdata, reqtype)
@@ -274,7 +237,6 @@ class KaisaGateway():
         resp_data = response.json()
         return status_code, resp_data
 
-
     def _market_req_decorate(self, reqdata):
 
         reqdata['headers'] = {
@@ -303,7 +265,6 @@ class KaisaGateway():
 
         return contractdata_all
 
-
     def query_symbollist(self, beginpos=0, count=1000):
 
         self.req_id += 1
@@ -328,7 +289,6 @@ class KaisaGateway():
             symbols = None
         return symbols
 
-
     def query_history_kline(self, symbol='01638', market='HK', klinetype='1d', adjfactortype='0', pagenum=1, pagesize=100):
         """
         query history kline (only for hk market)
@@ -340,7 +300,7 @@ class KaisaGateway():
                              '15min': 5,
                              '30min': 6,
                              '1hour': 7,
-                             '1day': 10
+                             '1d': 10
                              }
         klineid = klinetype2klineid.get(klinetype)
         rqdata = {
@@ -364,11 +324,68 @@ class KaisaGateway():
         if resp_status:
             body_data = resp_data['body']['records']
         else:
-            body_data = {'retCode': resp_data['retCode'],
-                         'retMsg': resp_data['retMsg']
-                         }
+            body_data = {
+                'retCode': resp_data['retCode'],
+                'retMsg': resp_data['retMsg']
+                }
         return resp_status, body_data
 
+    def get_history_kline_data(self, symbol=['00700'], start_date=None, end_date=None, unit='1d', market=1, fre='pre'):
+       '''
+       Args:
+            symbol: 合约列表 list
+            market: 市场类型：1 CH/ 2 HK/ 3 USA
+            unit: 频率 默认为 '1d' 将支持如下周期: '1m', '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
+            start_date: 开始日期
+            end_date: 结束日期
+            fre:是否复权  1 不复权/2 前复权/3 后复权
+        Returns: 返回历史日K/分钟K数据
+       '''
+        # /stock/quote/tickHistory/v2
+        # quoteInfo/kLineHistory/v2
+       post_url = HK_STOCK_KLINE_POST_URL
+       if unit != '1m' or unit != '1d':return '您好, 该接口暂时只提供日K和1分钟K线数据, 其他频率的数据, 接口维护团队会尽快补上!'
+       if unit == '1m':
+           if len(symbol) >1:return '您好, 检测到您填入多个合约获取历史分钟k线, 请修改后重新请求数据!'
+       if start_date is None and end_date is None:return '您好, 检测到您填入两个为None的日期, 请修改后重新请求数据!'
+       if market != 1:return '您好, 该接口暂时只提供港股数据, 请修改后重新请求数据!'
+       kprotoc.get_kline_hist_data_proto()
+
+
+    def get_history_broker_data(self, symbol='00700', start_date=None, end_date=None, market=1):
+        '''
+        Args:
+            symbol: 合约代码
+            market: 市场类型：1 CH/ 2 HK/ 3 USA
+            start_date: 开始日期
+            end_date: 结束日期
+        Returns: 返回历史经纪商队列数据
+        '''
+
+
+
+    def get_history_tick_data(self, symbol='00700', start_date=None, end_date=None, market=1):
+        '''
+        Args:
+            symbol: 合约代码
+            market: 市场类型：1 CH/ 2 HK/ 3 USA
+            start_date: 开始日期
+            end_date: 结束日期
+        Returns: 返回历史tick数据
+        '''
+        pass
+
+    def get_history_order_stall_data(self, symbol='00700', start_date=None, end_date=None, market=1):
+        '''
+        Args:
+            symbol: 合约代码
+            market: 市场类型：1 CH/ 2 HK/ 3 USA
+            start_date: 开始日期
+            end_date: 结束日期
+        Returns: 返回历史十档盘口数据
+        '''
+        URL = "https://sit-kgl.jt00000.com/kgl-stock-support-provider" + "/quoteInfo/order_stall"
+
 
     def query_history_tick(self, symbol='00700', market='HK', pagesize=10, pagenum=1):
 
@@ -397,9 +414,8 @@ class KaisaGateway():
                          }
         return resp_status, body_data
 
-
     def query_detailstate(self, symbol='01638', market='HK'):
-
+        # detailstate
         post_url = MARKET_POST_URL+ "/quoteInfo/todayDetailState"
         rqdata = {
             'code': symbol,
@@ -429,10 +445,10 @@ class KaisaGateway():
                          }
         return resp_status, body_data
 
-
-    def market_connect_ws(self):
+    def market_connect_ws0(self):
 
         threading.Thread(target=self.__market_connect_ws, daemon=True).start()
+        #self.__market_connect_ws()
 
     def get_market_connect_ws_status(self):
 
@@ -443,26 +459,37 @@ class KaisaGateway():
         if self._market_connect_status:
             return
 
+        # header = {"Authorization": self.gatewayToken,
+        #           "deviceId":"xxx",
+        #           "platform":"open",
+        #           "srvType":"open"}
+
+        data = {"deviceId": "xxx",
+                  "platform": "open",
+                  "srvType": "open"}
+
         header = {"Authorization": self.gatewayToken,
-                  "deviceId":"xxx",
-                  "platform":"open"}
+                  "headerParam": json.dumps(data)
+                    }
 
-        self._market_ws = websocket.create_connection(MARKET_WS_URL, header=header)
-        self._market_ping_th = threading.Thread(target=self._market_ping, args=(self._market_ws,), daemon=True)
-        self._market_ping_th.start()
+        try:
+            self._market_ws = websocket.create_connection(MARKET_WS_URL, header=header)
+            self._market_ping_th = threading.Thread(target=self._market_ping, args=(self._market_ws,), daemon=True)
+            self._market_ping_th.start()
 
-        self._market_connect_status = True
-        self._market_connect_msg = ""
+            self._market_connect_status = True
+            self._market_connect_msg = ""
 
-        print('wait to receive.')
-        while True:
-
-            text = self._market_ws.recv()
-            data = kprotoc.rec_bstring_to_json(text)
-            if data is None:
-                continue
-            self.on_market_data(data)
+            print('wait to receive.')
+            while True:
 
+                text = self._market_ws.recv()              # 接口实时返回的字符串
+                data = kprotoc.rec_bstring_to_json(text)   # 把字符串转换为json数据类型
+                if data is None:
+                    continue
+                self.on_market_data(data)
+        except Exception as exp:
+            print('__market_connect_ws:exp:{}'.format(exp))
 
     def on_market_packet(self, packet):
 
@@ -511,7 +538,6 @@ class KaisaGateway():
         }
         return req
 
-
     def _trade_req_decorate(self, reqdata, reqtype="normal"):
 
         json_dumps = lambda item: json.dumps(item, separators=(',', ':'))
@@ -646,6 +672,7 @@ class KaisaGateway():
                    }
 
         if method=="GET":
+            timestamp_ = generate_timestamp()
             header_ = reqdata['params'].copy()
             header_["Authorization"] = self.token
             headers["Sign"] = self.__sign(header_=header_, timestamp_=timestamp_, request_="GET")
@@ -671,7 +698,6 @@ class KaisaGateway():
 
         return reqdata
 
-
     def __sign(self, header_: dict = {}, request_=None, timestamp_=None):
 
         sign_ = None
@@ -682,7 +708,6 @@ class KaisaGateway():
         sign_ = kcrypto.encrypt_md5(str_)
         return sign_
 
-
     def trade_connect(self):
         """
         交易连接
@@ -706,7 +731,6 @@ class KaisaGateway():
         self.trade_connect_msg = ''
         return self.trade_connect_status, self.trade_connect_msg
 
-
     def trade_auth(self):
 
         data = {
@@ -730,7 +754,6 @@ class KaisaGateway():
         self.trade_auth_msg = ''
         return self.trade_auth_status, self.trade_auth_msg
 
-
     def trade_connect_ws(self):
         threading.Thread(target=self._start_trade_ws, daemon=True).start()
 
@@ -753,7 +776,6 @@ class KaisaGateway():
         """
         pass
 
-
     def _start_trade_ws(self):
 
         if self._ws_trade_connect_status:
@@ -771,7 +793,7 @@ class KaisaGateway():
 
         host = self.WEBSOCKET_TRADE_HOST
         header = {"Authorization": self.token}
-
+        # # create_connection--
         self._trade_ws = websocket.create_connection(host, header=header)
 
         req = self.generate_req(LOGIN, data)
@@ -786,7 +808,6 @@ class KaisaGateway():
             text = self._trade_ws.recv()
             self.on_trade_packet(text)
 
-
     def on_trade_packet(self, packet):
 
         if len(packet)==0:
@@ -800,7 +821,6 @@ class KaisaGateway():
             data = json.loads(packet)
             self.on_trade_data(packet)
 
-
     def send_order(self, bsFlag='B', price=550, qty=600, code='00700'):
 
         return self.place_order(bsFlag=bsFlag, price=price, qty=qty, code=code)
@@ -830,7 +850,6 @@ class KaisaGateway():
                          }
         return resp_status, body_data
 
-
     def cancel_order(self, sys_orderid):
 
         data = {
@@ -854,7 +873,6 @@ class KaisaGateway():
                          }
         return resp_status, body_data
 
-
     def query_portfolio(self):
 
         reqdata = {"method":"GET",
@@ -872,7 +890,6 @@ class KaisaGateway():
                          }
         return resp_status, body_data
 
-
     def query_balance(self):
 
         reqdata = {"method": "GET",
@@ -889,7 +906,6 @@ class KaisaGateway():
                          }
         return resp_status, body_data
 
-
     def query_position(self):
 
         reqdata = {"method": "GET",
@@ -906,7 +922,6 @@ class KaisaGateway():
                          }
         return resp_status, body_data
 
-
     def query_order(self):
 
         reqdata = {"method": "GET",
